@@ -28,6 +28,7 @@ from base64 import b64decode, b64encode
 import calendar
 import datetime
 import decimal
+import email
 from enum import Enum
 import json
 import logging
@@ -78,6 +79,32 @@ class UTC(datetime.tzinfo):
         """No daylight saving for UTC."""
         return datetime.timedelta(hours=1)
 
+try:
+    from datetime import timezone as _FixedOffset
+except ImportError:  # Python 2.7
+    class _FixedOffset(datetime.tzinfo):  # type: ignore
+        """Fixed offset in minutes east from UTC.
+        Copy/pasted from Python doc
+        :param datetime.timedelta offset: offset in timedelta format
+        """
+
+        def __init__(self, offset):
+            self.__offset = offset
+
+        def utcoffset(self, dt):
+            return self.__offset
+
+        def tzname(self, dt):
+            return str(self.__offset.total_seconds()/3600)
+
+        def __repr__(self):
+            return "<FixedOffset {}>".format(self.tzname(None))
+
+        def dst(self, dt):
+            return datetime.timedelta(0)
+
+        def __getinitargs__(self):
+            return (self.__offset,)
 
 try:
     from datetime import timezone
@@ -188,12 +215,12 @@ class Model(object):
 
     @classmethod
     def _create_xml_node(cls):
-        """Create XML node from "_xml_map".
+        """Create XML node.
         """
         try:
             xml_map = cls._xml_map
         except AttributeError:
-            raise ValueError("This model has no XML definition")
+            xml_map = {}
 
         return _create_xml_node(
             xml_map.get('name', cls.__name__),
@@ -217,6 +244,9 @@ class Model(object):
 
             try:
                 debug_name = "{}.{}".format(self.__class__.__name__, attr_name)
+                # https://github.com/Azure/msrest-for-python/issues/85
+                if value is not None and attr_type in Serializer.basic_types.values():
+                    value = Serializer.serialize_basic(value, attr_type)
                 Serializer.validate(value, debug_name, **self._validation.get(attr_name, {}))
             except ValidationError as validation_error:
                 validation_result.append(validation_error)
@@ -224,19 +254,21 @@ class Model(object):
             validation_result += _recursive_validate(attr_name, attr_type, value)
         return validation_result
 
-    def serialize(self, keep_readonly=False):
+    def serialize(self, keep_readonly=False, **kwargs):
         """Return the JSON that would be sent to azure from this model.
 
         This is an alias to `as_dict(full_restapi_key_transformer, keep_readonly=False)`.
 
+        If you want XML serialization, you can pass the kwargs is_xml=True.
+
         :param bool keep_readonly: If you want to serialize the readonly attributes
         :returns: A dict JSON compatible object
         :rtype: dict
         """
         serializer = Serializer(self._infer_class_models())
-        return serializer._serialize(self, keep_readonly=keep_readonly)
+        return serializer._serialize(self, keep_readonly=keep_readonly, **kwargs)
 
-    def as_dict(self, keep_readonly=True, key_transformer=attribute_transformer):
+    def as_dict(self, keep_readonly=True, key_transformer=attribute_transformer, **kwargs):
         """Return a dict that can be JSONify using json.dump.
 
         Advanced usage might optionaly use a callback as parameter:
@@ -261,12 +293,14 @@ class Model(object):
         - full_restapi_key_transformer
         - last_restapi_key_transformer
 
+        If you want XML serialization, you can pass the kwargs is_xml=True.
+
         :param function key_transformer: A key transformer function.
         :returns: A dict JSON compatible object
         :rtype: dict
         """
         serializer = Serializer(self._infer_class_models())
-        return serializer._serialize(self, key_transformer=key_transformer, keep_readonly=keep_readonly)
+        return serializer._serialize(self, key_transformer=key_transformer, keep_readonly=keep_readonly, **kwargs)
 
     @classmethod
     def _infer_class_models(cls):
@@ -308,8 +342,8 @@ class Model(object):
         """
         deserializer = Deserializer(cls._infer_class_models())
         deserializer.key_extractors = [
-            rest_key_case_insensitive_extractor,
             attribute_key_case_insensitive_extractor,
+            rest_key_case_insensitive_extractor,
             last_rest_key_case_insensitive_extractor
         ] if key_extractors is None else key_extractors
         return deserializer(cls.__name__, data, content_type=content_type)
@@ -332,8 +366,15 @@ class Model(object):
         for subtype_key in cls.__dict__.get('_subtype_map', {}).keys():
             subtype_value = None
 
-            rest_api_response_key = cls._get_rest_key_parts(subtype_key)[-1]
-            subtype_value = response.pop(rest_api_response_key, None) or response.pop(subtype_key, None)
+            if not isinstance(response, ET.Element):
+                rest_api_response_key = cls._get_rest_key_parts(subtype_key)[-1]
+                subtype_value = response.pop(rest_api_response_key, None) or response.pop(subtype_key, None)
+            else:
+                subtype_value = xml_key_extractor(
+                    subtype_key,
+                    cls._attribute_map[subtype_key],
+                    response
+                )
             if subtype_value:
                 # Try to match base class. Can be class name only
                 # (bug to fix in Autorest to support x-ms-discriminator-name)
@@ -409,6 +450,7 @@ class Serializer(object):
             'unix-time': Serializer.serialize_unix,
             'duration': Serializer.serialize_duration,
             'date': Serializer.serialize_date,
+            'time': Serializer.serialize_time,
             'decimal': Serializer.serialize_decimal,
             'long': Serializer.serialize_long,
             'bytearray': Serializer.serialize_bytearray,
@@ -482,11 +524,19 @@ class Serializer(object):
 
                     ### Incorporate this data in the right place ###
                     if is_xml_model_serialization:
-                        xml_desc = attr_desc['xml']
-                        xml_name = xml_desc['name']
-                        if "attr" in xml_desc and xml_desc["attr"]:
+                        xml_desc = attr_desc.get('xml', {})
+                        xml_name = xml_desc.get('name', attr_desc['key'])
+                        xml_prefix = xml_desc.get('prefix', None)
+                        xml_ns = xml_desc.get('ns', None)
+                        if xml_desc.get("attr", False):
+                            if xml_ns:
+                                ET.register_namespace(xml_prefix, xml_ns)
+                                xml_name = "{{{}}}{}".format(xml_ns, xml_name)
                             serialized.set(xml_name, new_attr)
                             continue
+                        if xml_desc.get("text", False):
+                            serialized.text = new_attr
+                            continue
                         if isinstance(new_attr, list):
                             serialized.extend(new_attr)
                         elif isinstance(new_attr, ET.Element):
@@ -502,8 +552,8 @@ class Serializer(object):
                             # Integrate namespace if necessary
                             local_node = _create_xml_node(
                                 xml_name,
-                                xml_desc.get('prefix', None),
-                                xml_desc.get('ns', None)
+                                xml_prefix,
+                                xml_ns
                             )
                             local_node.text = unicode_str(new_attr)
                             serialized.append(local_node)
@@ -578,6 +628,14 @@ class Serializer(object):
                 raise errors[0]
         return self._serialize(data, data_type, **kwargs)
 
+    def _http_component_validation(self, data, data_type, name, **kwargs):
+        if self.client_side_validation:
+            # https://github.com/Azure/msrest-for-python/issues/85
+            if data is not None and data_type in self.basic_types.values():
+                data = self.serialize_basic(data, data_type, **kwargs)
+            data = self.validate(data, name, required=True, **kwargs)
+        return data
+
     def url(self, name, data, data_type, **kwargs):
         """Serialize data intended for a URL path.
 
@@ -587,8 +645,7 @@ class Serializer(object):
         :raises: TypeError if serialization fails.
         :raises: ValueError if data is None
         """
-        if self.client_side_validation:
-            data = self.validate(data, name, required=True, **kwargs)
+        data = self._http_component_validation(data, data_type, name, **kwargs)
         try:
             output = self.serialize_data(data, data_type, **kwargs)
             if data_type == 'bool':
@@ -612,12 +669,25 @@ class Serializer(object):
         :raises: TypeError if serialization fails.
         :raises: ValueError if data is None
         """
-        if self.client_side_validation:
-            data = self.validate(data, name, required=True, **kwargs)
+        data = self._http_component_validation(data, data_type, name, **kwargs)
         try:
-            if data_type in ['[str]']:
-                data = ["" if d is None else d for d in data]
+            # Treat the list aside, since we don't want to encode the div separator
+            if data_type.startswith("["):
+                internal_data_type = data_type[1:-1]
+                data = [
+                    self.serialize_data(d, internal_data_type, **kwargs) if d is not None else ""
+                    for d
+                    in data
+                ]
+                if not kwargs.get('skip_quote', False):
+                    data = [
+                        quote(str(d), safe='')
+                        for d
+                        in data
+                    ]
+                return str(self.serialize_iter(data, internal_data_type, **kwargs))
 
+            # Not a list, regular serialization
             output = self.serialize_data(data, data_type, **kwargs)
             if data_type == 'bool':
                 output = json.dumps(output)
@@ -639,8 +709,7 @@ class Serializer(object):
         :raises: TypeError if serialization fails.
         :raises: ValueError if data is None
         """
-        if self.client_side_validation:
-            data = self.validate(data, name, required=True, **kwargs)
+        data = self._http_component_validation(data, data_type, name, **kwargs)
         try:
             if data_type in ['[str]']:
                 data = ["" if d is None else d for d in data]
@@ -713,14 +782,16 @@ class Serializer(object):
         else:
             return self._serialize(data, **kwargs)
 
-    def _get_custom_serializers(self, data_type, **kwargs):
+    @classmethod
+    def _get_custom_serializers(cls, data_type, **kwargs):
         custom_serializer = kwargs.get("basic_types_serializers", {}).get(data_type)
         if custom_serializer:
             return custom_serializer
         if kwargs.get("is_xml", False):
-            return self._xml_basic_types_serializers.get(data_type)
+            return cls._xml_basic_types_serializers.get(data_type)
 
-    def serialize_basic(self, data, data_type, **kwargs):
+    @classmethod
+    def serialize_basic(cls, data, data_type, **kwargs):
         """Serialize basic builting data type.
         Serializes objects to str, int, float or bool.
 
@@ -731,14 +802,15 @@ class Serializer(object):
         :param data: Object to be serialized.
         :param str data_type: Type of object in the iterable.
         """
-        custom_serializer = self._get_custom_serializers(data_type, **kwargs)
+        custom_serializer = cls._get_custom_serializers(data_type, **kwargs)
         if custom_serializer:
             return custom_serializer(data)
         if data_type == 'str':
-            return self.serialize_unicode(data)
+            return cls.serialize_unicode(data)
         return eval(data_type)(data)
 
-    def serialize_unicode(self, data):
+    @classmethod
+    def serialize_unicode(cls, data):
         """Special handling for serializing unicode strings in Py2.
         Encode to UTF-8 if unicode, otherwise handle as a str.
 
@@ -764,7 +836,9 @@ class Serializer(object):
         """Serialize iterable.
 
         Supported kwargs:
-        serialization_ctxt dict : The current entry of _attribute_map, or same format. serialization_ctxt['type'] should be same as data_type.
+        - serialization_ctxt dict : The current entry of _attribute_map, or same format.
+          serialization_ctxt['type'] should be same as data_type.
+        - is_xml bool : If set, serialize as XML
 
         :param list attr: Object to be serialized.
         :param str iter_type: Type of object in the iterable.
@@ -778,6 +852,7 @@ class Serializer(object):
             raise SerializationError("Refuse str type as a valid iter type.")
 
         serialization_ctxt = kwargs.get("serialization_ctxt", {})
+        is_xml = kwargs.get("is_xml", False)
 
         serialized = []
         for d in data:
@@ -790,13 +865,15 @@ class Serializer(object):
             serialized = ['' if s is None else str(s) for s in serialized]
             serialized = div.join(serialized)
 
-        if 'xml' in serialization_ctxt:
+        if 'xml' in serialization_ctxt or is_xml:
             # XML serialization is more complicated
-            xml_desc = serialization_ctxt['xml']
-            xml_name = xml_desc['name']
+            xml_desc = serialization_ctxt.get('xml', {})
+            xml_name = xml_desc.get('name')
+            if not xml_name:
+                xml_name = serialization_ctxt['key']
 
             # Create a wrap node if necessary (use the fact that Element and list have "append")
-            is_wrapped = "wrapped" in xml_desc and xml_desc["wrapped"]
+            is_wrapped = xml_desc.get("wrapped", False)
             node_name = xml_desc.get("itemsName", xml_name)
             if is_wrapped:
                 final_result = _create_xml_node(
@@ -874,9 +951,21 @@ class Serializer(object):
             return self.serialize_basic(attr, self.basic_types[obj_type], **kwargs)
         if obj_type is _long_type:
             return self.serialize_long(attr)
+        if obj_type is unicode_str:
+            return self.serialize_unicode(attr)
+        if obj_type is datetime.datetime:
+            return self.serialize_iso(attr)
+        if obj_type is datetime.date:
+            return self.serialize_date(attr)
+        if obj_type is datetime.time:
+            return self.serialize_time(attr)
+        if obj_type is datetime.timedelta:
+            return self.serialize_duration(attr)
+        if obj_type is decimal.Decimal:
+            return self.serialize_decimal(attr)
 
         # If it's a model or I know this dependency, serialize as a Model
-        elif obj_type in self.dependencies.values() or isinstance(obj_type, Model):
+        elif obj_type in self.dependencies.values() or isinstance(attr, Model):
             return self._serialize(attr)
 
         if obj_type == dict:
@@ -965,6 +1054,20 @@ class Serializer(object):
         t = "{:04}-{:02}-{:02}".format(attr.year, attr.month, attr.day)
         return t
 
+    @staticmethod
+    def serialize_time(attr, **kwargs):
+        """Serialize Time object into ISO-8601 formatted string.
+
+        :param datetime.time attr: Object to be serialized.
+        :rtype: str
+        """
+        if isinstance(attr, str):
+            attr = isodate.parse_time(attr)
+        t = "{:02}:{:02}:{:02}".format(attr.hour, attr.minute, attr.second)
+        if attr.microsecond:
+            t += ".{:02}".format(attr.microsecond)
+        return t
+
     @staticmethod
     def serialize_duration(attr, **kwargs):
         """Serialize TimeDelta object into ISO-8601 formatted string.
@@ -1059,6 +1162,11 @@ def rest_key_extractor(attr, attr_desc, data):
             break
         working_key = _decode_attribute_map_key(dict_keys[0])
         working_data = working_data.get(working_key, data)
+        if working_data is None:
+            # If at any point while following flatten JSON path see None, it means
+            # that all properties under are None as well
+            # https://github.com/Azure/msrest-for-python/issues/197
+            return None
         key = '.'.join(dict_keys[1:])
 
     return working_data.get(key)
@@ -1074,17 +1182,28 @@ def rest_key_case_insensitive_extractor(attr, attr_desc, data):
             break
         working_key = _decode_attribute_map_key(dict_keys[0])
         working_data = attribute_key_case_insensitive_extractor(working_key, None, working_data)
+        if working_data is None:
+            # If at any point while following flatten JSON path see None, it means
+            # that all properties under are None as well
+            # https://github.com/Azure/msrest-for-python/issues/197
+            return None
         key = '.'.join(dict_keys[1:])
 
     if working_data:
         return attribute_key_case_insensitive_extractor(key, None, working_data)
 
 def last_rest_key_extractor(attr, attr_desc, data):
+    """Extract the attribute in "data" based on the last part of the JSON path key.
+    """
     key = attr_desc['key']
     dict_keys = _FLATTEN.split(key)
     return attribute_key_extractor(dict_keys[-1], None, data)
 
 def last_rest_key_case_insensitive_extractor(attr, attr_desc, data):
+    """Extract the attribute in "data" based on the last part of the JSON path key.
+
+    This is the case insensitive version of "last_rest_key_extractor"
+    """
     key = attr_desc['key']
     dict_keys = _FLATTEN.split(key)
     return attribute_key_case_insensitive_extractor(dict_keys[-1], None, data)
@@ -1102,53 +1221,68 @@ def attribute_key_case_insensitive_extractor(attr, _, data):
 
     return data.get(found_key)
 
-def xml_key_extractor(attr, attr_desc, data):
-    # Test if this model is XML ready first
-    if 'xml' not in attr_desc:
-        return None
+def _extract_name_from_internal_type(internal_type):
+    """Given an internal type XML description, extract correct XML name with namespace.
 
+    :param dict internal_type: An model type
+    :rtype: tuple
+    :returns: A tuple XML name + namespace dict
+    """
+    internal_type_xml_map = getattr(internal_type, "_xml_map", {})
+    xml_name = internal_type_xml_map.get('name', internal_type.__name__)
+    xml_ns = internal_type_xml_map.get("ns", None)
+    if xml_ns:
+        xml_name = "{{{}}}{}".format(xml_ns, xml_name)
+    return xml_name
+
+
+def xml_key_extractor(attr, attr_desc, data):
     if isinstance(data, dict):
         return None
 
-    xml_desc = attr_desc['xml']
-    xml_name = xml_desc['name']
-    xml_ns = xml_desc.get('ns', None)
+    # Test if this model is XML ready first
+    if not isinstance(data, ET.Element):
+        return None
 
-    # If it's an attribute, that's simple
-    if "attr" in xml_desc and xml_desc["attr"]:
-        return data.get(xml_name)
-
-    # Integrate namespace if necessary
-    if xml_ns:
-        ns = {'prefix': xml_ns}
-        xml_name = "prefix:"+xml_name
-    else:
-        ns = {} # And keep same xml_name
+    xml_desc = attr_desc.get('xml', {})
+    xml_name = xml_desc.get('name', attr_desc['key'])
 
     # Look for a children
     is_iter_type = attr_desc['type'].startswith("[")
-    is_wrapped = "wrapped" in xml_desc and xml_desc["wrapped"]
+    is_wrapped = xml_desc.get("wrapped", False)
     internal_type = attr_desc.get("internalType", None)
+    internal_type_xml_map = getattr(internal_type, "_xml_map", {})
+
+    # Integrate namespace if necessary
+    xml_ns = xml_desc.get('ns', internal_type_xml_map.get("ns", None))
+    if xml_ns:
+        xml_name = "{{{}}}{}".format(xml_ns, xml_name)
+
+    # If it's an attribute, that's simple
+    if xml_desc.get("attr", False):
+        return data.get(xml_name)
+
+    # If it's x-ms-text, that's simple too
+    if xml_desc.get("text", False):
+        return data.text
 
     # Scenario where I take the local name:
     # - Wrapped node
     # - Internal type is an enum (considered basic types)
     # - Internal type has no XML/Name node
-    if is_wrapped or (internal_type and (issubclass(internal_type, Enum) or 'name' not in internal_type._xml_map)):
-        children = data.findall(xml_name, ns)
+    if is_wrapped or (internal_type and (issubclass(internal_type, Enum) or 'name' not in internal_type_xml_map)):
+        children = data.findall(xml_name)
     # If internal type has a local name and it's not a list, I use that name
-    elif not is_iter_type and internal_type and 'name' in internal_type._xml_map:
-        xml_name = internal_type._xml_map["name"]
-        ns = internal_type._xml_map.get("ns", None)
-        children = data.findall(xml_name, ns)
+    elif not is_iter_type and internal_type and 'name' in internal_type_xml_map:
+        xml_name = _extract_name_from_internal_type(internal_type)
+        children = data.findall(xml_name)
     # That's an array
     else:
         if internal_type: # Complex type, ignore itemsName and use the complex type name
-            items_name = internal_type._xml_map["name"]
-            ns = internal_type._xml_map.get("ns", None)
+            items_name = _extract_name_from_internal_type(internal_type)
         else:
             items_name = xml_desc.get("itemsName", xml_name)
-        children = data.findall(items_name, ns)
+        children = data.findall(items_name)
 
     if len(children) == 0:
         if is_iter_type:
@@ -1178,8 +1312,8 @@ def xml_key_extractor(attr, attr_desc, data):
 class Deserializer(object):
     """Response object model deserializer.
 
-    :param dict classes: Class type dictionary for deserializing
-     complex types.
+    :param dict classes: Class type dictionary for deserializing complex types.
+    :ivar list key_extractors: Ordered list of extractors to be used by this deserializer.
     """
 
     basic_types = {str: 'str', int: 'int', bool: 'bool', float: 'float'}
@@ -1195,6 +1329,7 @@ class Deserializer(object):
             'unix-time': Deserializer.deserialize_unix,
             'duration': Deserializer.deserialize_duration,
             'date': Deserializer.deserialize_date,
+            'time': Deserializer.deserialize_time,
             'decimal': Deserializer.deserialize_decimal,
             'long': Deserializer.deserialize_long,
             'bytearray': Deserializer.deserialize_bytearray,
@@ -1293,7 +1428,15 @@ class Deserializer(object):
                     found_value = key_extractor(attr, attr_desc, data)
                     if found_value is not None:
                         if raw_value is not None and raw_value != found_value:
-                            raise KeyError('Use twice the key: "{}"'.format(attr))
+                            msg = ("Ignoring extracted value '%s' from %s for key '%s'"
+                                   " (duplicate extraction, follow extractors order)" )
+                            _LOGGER.warning(
+                                msg,
+                                found_value,
+                                key_extractor,
+                                attr
+                            )
+                            continue
                         raw_value = found_value
 
                 value = self.deserialize_data(raw_value, attr_desc['type'])
@@ -1343,6 +1486,26 @@ class Deserializer(object):
             pass  # Target is not a Model, no classify
         return target, target.__class__.__name__
 
+    def failsafe_deserialize(self, target_obj, data, content_type=None):
+        """Ignores any errors encountered in deserialization,
+        and falls back to not deserializing the object. Recommended
+        for use in error deserialization, as we want to return the
+        HttpResponseError to users, and not have them deal with
+        a deserialization error.
+
+        :param str target_obj: The target object type to deserialize to.
+        :param str/dict data: The response data to deseralize.
+        :param str content_type: Swagger "produces" if available.
+        """
+        try:
+            return self(target_obj, data, content_type=content_type)
+        except:
+            _LOGGER.warning(
+                "Ran into a deserialization error. Ignoring since this is failsafe deserialization",
+				exc_info=True
+            )
+            return None
+
     @staticmethod
     def _unpack_content(raw_data, content_type=None):
         """Extract the correct structure for deserialization.
@@ -1728,6 +1891,20 @@ class Deserializer(object):
         # This must NOT use defaultmonth/defaultday. Using None ensure this raises an exception.
         return isodate.parse_date(attr, defaultmonth=None, defaultday=None)
 
+    @staticmethod
+    def deserialize_time(attr):
+        """Deserialize ISO-8601 formatted string into time object.
+
+        :param str attr: response string to be deserialized.
+        :rtype: datetime.time
+        :raises: DeserializationError if string format invalid.
+        """
+        if isinstance(attr, ET.Element):
+            attr = attr.text
+        if re.search(r"[^\W\d_]", attr, re.I + re.U):
+            raise DeserializationError("Date must have only digits and -. Received: %s" % attr)
+        return isodate.parse_time(attr)
+
     @staticmethod
     def deserialize_rfc(attr):
         """Deserialize RFC-1123 formatted string into Datetime object.
@@ -1739,10 +1916,13 @@ class Deserializer(object):
         if isinstance(attr, ET.Element):
             attr = attr.text
         try:
-            date_obj = datetime.datetime.strptime(
-                attr, "%a, %d %b %Y %H:%M:%S %Z")
+            parsed_date = email.utils.parsedate_tz(attr)
+            date_obj = datetime.datetime(
+                *parsed_date[:6],
+                tzinfo=_FixedOffset(datetime.timedelta(minutes=(parsed_date[9] or 0)/60))
+            )
             if not date_obj.tzinfo:
-                date_obj = date_obj.replace(tzinfo=TZ_UTC)
+                date_obj = date_obj.astimezone(tz=TZ_UTC)
         except ValueError as err:
             msg = "Cannot deserialize to rfc datetime object."
             raise_with_traceback(DeserializationError, msg, err)
