@@ -102,6 +102,26 @@ class TestXmlDeserialization:
 
         assert result.language == u"français"
 
+    def test_basic_text(self):
+        """Test a XML with unicode."""
+        basic_xml = u"""<?xml version="1.0" encoding="utf-8"?>
+            <Data language="english">I am text</Data>"""
+
+        class XmlModel(Model):
+            _attribute_map = {
+                'language': {'key': 'language', 'type': 'str', 'xml':{'name': 'language', 'attr': True}},
+                'content': {'key': 'content', 'type': 'str', 'xml':{'text': True}},
+            }
+            _xml_map = {
+                'name': 'Data'
+            }
+
+        s = Deserializer({"XmlModel": XmlModel})
+        result = s(XmlModel, basic_xml, "application/xml")
+
+        assert result.language == "english"
+        assert result.content == "I am text"
+
     def test_add_prop(self):
         """Test addProp as a dict.
         """
@@ -471,6 +491,138 @@ class TestXmlDeserialization:
 
         assert result.age == 37
 
+    def test_complex_namespace(self):
+        """Test recursive namespace."""
+        basic_xml = """<?xml version="1.0"?>
+            <entry xmlns="http://www.w3.org/2005/Atom" xmlns:i="http://www.w3.org/2001/XMLSchema-instance">
+                <author>
+                    <name>lmazuel</name>
+                </author>
+                <AuthorizationRules xmlns="http://schemas.microsoft.com/netservices/2010/10/servicebus/connect">
+                    <AuthorizationRule i:type="SharedAccessAuthorizationRule">
+                        <KeyName>testpolicy</KeyName>
+                    </AuthorizationRule>
+                </AuthorizationRules>
+                <CountDetails xmlns="http://schemas.microsoft.com/netservices/2010/10/servicebus/connect">
+                    <d2p1:ActiveMessageCount xmlns:d2p1="http://schemas.microsoft.com/netservices/2011/06/servicebus">12</d2p1:ActiveMessageCount>
+                </CountDetails>
+            </entry>"""
+
+        class XmlRoot(Model):
+            _attribute_map = {
+                'author': {'key': 'author', 'type': 'QueueDescriptionResponseAuthor'},
+                'authorization_rules': {'key': 'AuthorizationRules', 'type': '[AuthorizationRule]', 'xml': {'ns': 'http://schemas.microsoft.com/netservices/2010/10/servicebus/connect', 'wrapped': True, 'itemsNs': 'http://schemas.microsoft.com/netservices/2010/10/servicebus/connect'}},
+                'message_count_details': {'key': 'MessageCountDetails', 'type': 'MessageCountDetails'},
+            }
+            _xml_map = {
+                'name': 'entry', 'ns': 'http://www.w3.org/2005/Atom'
+            }
+
+        class QueueDescriptionResponseAuthor(Model):
+            _attribute_map = {
+                'name': {'key': 'name', 'type': 'str', 'xml': {'ns': 'http://www.w3.org/2005/Atom'}},
+            }
+            _xml_map = {
+                'ns': 'http://www.w3.org/2005/Atom'
+            }
+
+        class AuthorizationRule(Model):
+            _attribute_map = {
+                'type': {'key': 'type', 'type': 'str', 'xml': {'attr': True, 'prefix': 'i', 'ns': 'http://www.w3.org/2001/XMLSchema-instance'}},
+                'key_name': {'key': 'KeyName', 'type': 'str', 'xml': {'ns': 'http://schemas.microsoft.com/netservices/2010/10/servicebus/connect'}},
+            }
+            _xml_map = {
+                'ns': 'http://schemas.microsoft.com/netservices/2010/10/servicebus/connect'
+            }
+
+        class MessageCountDetails(Model):
+            _attribute_map = {
+                'active_message_count': {'key': 'ActiveMessageCount', 'type': 'int', 'xml': {'prefix': 'd2p1', 'ns': 'http://schemas.microsoft.com/netservices/2011/06/servicebus'}},
+            }
+            _xml_map = {
+                'name': 'CountDetails', 'ns': 'http://schemas.microsoft.com/netservices/2010/10/servicebus/connect'
+            }
+
+
+        s = Deserializer({
+            "XmlRoot": XmlRoot,
+            "QueueDescriptionResponseAuthor": QueueDescriptionResponseAuthor,
+            "AuthorizationRule": AuthorizationRule,
+            "MessageCountDetails": MessageCountDetails,
+        })
+        result = s(XmlRoot, basic_xml, "application/xml")
+
+        assert result.author.name == "lmazuel"
+        assert result.authorization_rules[0].key_name == "testpolicy"
+        assert result.authorization_rules[0].type == "SharedAccessAuthorizationRule"
+        assert result.message_count_details.active_message_count == 12
+
+    def test_polymorphic_deserialization(self):
+
+        basic_xml = """<?xml version="1.0"?>
+            <entry xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
+                <Filter xsi:type="CorrelationFilter">
+                    <CorrelationId>12</CorrelationId>
+                </Filter>
+            </entry>"""
+
+        class XmlRoot(Model):
+            _attribute_map = {
+                'filter': {'key': 'Filter', 'type': 'RuleFilter'},
+            }
+            _xml_map = {
+                'name': 'entry'
+            }
+
+        class RuleFilter(Model):
+            _attribute_map = {
+                'type': {'key': 'type', 'type': 'str', 'xml': {'attr': True, 'prefix': 'xsi', 'ns': 'http://www.w3.org/2001/XMLSchema-instance'}},
+            }
+
+            _subtype_map = {
+                'type': {'CorrelationFilter': 'CorrelationFilter', 'SqlFilter': 'SqlFilter'}
+            }
+            _xml_map = {
+                'name': 'Filter'
+            }
+
+        class CorrelationFilter(RuleFilter):
+            _attribute_map = {
+                'type': {'key': 'type', 'type': 'str', 'xml': {'attr': True, 'prefix': 'xsi', 'ns': 'http://www.w3.org/2001/XMLSchema-instance'}},
+                'correlation_id': {'key': 'CorrelationId', 'type': 'int'},
+            }
+
+            def __init__(
+                self,
+                correlation_id = None,
+                **kwargs
+            ):
+                super(CorrelationFilter, self).__init__(**kwargs)
+                self.type = 'CorrelationFilter'
+                self.correlation_id = correlation_id
+
+        class SqlFilter(RuleFilter):
+            _attribute_map = {
+                'type': {'key': 'type', 'type': 'str', 'xml': {'attr': True, 'prefix': 'xsi', 'ns': 'http://www.w3.org/2001/XMLSchema-instance'}},
+            }
+
+            def __init__(
+                self,
+                **kwargs
+            ):
+                pytest.fail("Don't instantiate me")
+
+        s = Deserializer({
+            "XmlRoot": XmlRoot,
+            "RuleFilter": RuleFilter,
+            "SqlFilter": SqlFilter,
+            "CorrelationFilter": CorrelationFilter,
+        })
+        result = s(XmlRoot, basic_xml, "application/xml")
+
+        assert isinstance(result.filter, CorrelationFilter)
+        assert result.filter.correlation_id == 12
+
 
 class TestXmlSerialization:
 
@@ -621,6 +773,31 @@ class TestXmlSerialization:
 
         assert_xml_equals(rawxml, basic_xml)
 
+    def test_basic_text(self):
+        """Test a XML with unicode."""
+        basic_xml = ET.fromstring("""<?xml version="1.0" encoding="utf-8"?>
+            <Data language="english">I am text</Data>""")
+
+        class XmlModel(Model):
+            _attribute_map = {
+                'language': {'key': 'language', 'type': 'str', 'xml':{'name': 'language', 'attr': True}},
+                'content': {'key': 'content', 'type': 'str', 'xml':{'text': True}},
+            }
+            _xml_map = {
+                'name': 'Data'
+            }
+
+        mymodel = XmlModel(
+            language="english",
+            content="I am text"
+        )
+
+        s = Serializer({"XmlModel": XmlModel})
+        rawxml = s.body(mymodel, 'XmlModel')
+
+        assert_xml_equals(rawxml, basic_xml)
+
+
     def test_direct_array(self):
         """Test an ultra basic XML."""
         basic_xml = ET.fromstring("""<?xml version="1.0"?>
@@ -1359,4 +1536,63 @@ class TestXmlSerialization:
         s = Serializer({"XmlModel": XmlModel})
         rawxml = s.body(mymodel, 'XmlModel', is_xml=True)
 
-        assert_xml_equals(rawxml, basic_xml)
\ No newline at end of file
+        assert_xml_equals(rawxml, basic_xml)
+
+    @pytest.mark.skipif(sys.version_info < (3,6),
+                        reason="Unstable before python3.6 for some reasons")
+    def test_complex_namespace(self):
+        """Test recursive namespace."""
+        basic_xml = ET.fromstring("""<?xml version="1.0"?>
+            <entry xmlns="http://www.w3.org/2005/Atom" xmlns:i="http://www.w3.org/2001/XMLSchema-instance">
+                <author>
+                    <name>lmazuel</name>
+                </author>
+                <AuthorizationRules xmlns="http://schemas.microsoft.com/netservices/2010/10/servicebus/connect">
+                    <AuthorizationRule i:type="SharedAccessAuthorizationRule">
+                        <KeyName>testpolicy</KeyName>
+                    </AuthorizationRule>
+                </AuthorizationRules>
+            </entry>""")
+
+        class XmlRoot(Model):
+            _attribute_map = {
+                'author': {'key': 'author', 'type': 'QueueDescriptionResponseAuthor'},
+                'authorization_rules': {'key': 'AuthorizationRules', 'type': '[AuthorizationRule]', 'xml': {'ns': 'http://schemas.microsoft.com/netservices/2010/10/servicebus/connect', 'wrapped': True, 'itemsNs': 'http://schemas.microsoft.com/netservices/2010/10/servicebus/connect'}},
+            }
+            _xml_map = {
+                'name': 'entry', 'ns': 'http://www.w3.org/2005/Atom'
+            }
+
+        class QueueDescriptionResponseAuthor(Model):
+            _attribute_map = {
+                'name': {'key': 'name', 'type': 'str', 'xml': {'ns': 'http://www.w3.org/2005/Atom'}},
+            }
+            _xml_map = {
+                'ns': 'http://www.w3.org/2005/Atom'
+            }
+
+        class AuthorizationRule(Model):
+            _attribute_map = {
+                'type': {'key': 'type', 'type': 'str', 'xml': {'attr': True, 'prefix': 'i', 'ns': 'http://www.w3.org/2001/XMLSchema-instance'}},
+                'key_name': {'key': 'KeyName', 'type': 'str', 'xml': {'ns': 'http://schemas.microsoft.com/netservices/2010/10/servicebus/connect'}},
+            }
+            _xml_map = {
+                'ns': 'http://schemas.microsoft.com/netservices/2010/10/servicebus/connect'
+            }
+
+        mymodel = XmlRoot(
+            author = QueueDescriptionResponseAuthor(name = "lmazuel"),
+            authorization_rules = [AuthorizationRule(
+                type="SharedAccessAuthorizationRule",
+                key_name="testpolicy"
+            )]
+        )
+
+        s = Serializer({
+            "XmlRoot": XmlRoot,
+            "QueueDescriptionResponseAuthor": QueueDescriptionResponseAuthor,
+            "AuthorizationRule": AuthorizationRule,
+        })
+        rawxml = s.body(mymodel, 'XmlModel')
+
+        assert_xml_equals(rawxml, basic_xml)
